﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Analysis;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Optimization.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ALPS {
  [Item("EvaluatedSolutionsHistoryAnalyzer", "")]
  [StorableClass]
  public sealed class EvaluatedSolutionsHistoryAnalyzer : AlgorithmOperator, IAnalyzer {
    #region Parameter Properties
    public ILookupParameter<IntValue> EvaluatedSolutionsParameter {
      get { return (ILookupParameter<IntValue>)Parameters["EvaluatedSolutions"]; }
    }
    public ILookupParameter<DataTable> EvaluatedSolutionsHistoryParameter {
      get { return (ILookupParameter<DataTable>)Parameters["EvaluatedSolutionsHistory"]; }
    }
    public ValueLookupParameter<ResultCollection> ResultsParameter {
      get { return (ValueLookupParameter<ResultCollection>)Parameters["Results"]; }
    }
    #endregion

    public bool EnabledByDefault { get { return false; } }

    [StorableConstructor]
    private EvaluatedSolutionsHistoryAnalyzer(bool deserializing)
      : base(deserializing) { }
    private EvaluatedSolutionsHistoryAnalyzer(EvaluatedSolutionsHistoryAnalyzer original, Cloner cloner)
      : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new EvaluatedSolutionsHistoryAnalyzer(this, cloner);
    }
    public EvaluatedSolutionsHistoryAnalyzer()
      : base() {
      #region Create parameters
      Parameters.Add(new LookupParameter<IntValue>("EvaluatedSolutions", ""));
      Parameters.Add(new ValueLookupParameter<DataTable>("EvaluatedSolutionsHistory"));
      Parameters.Add(new ValueLookupParameter<ResultCollection>("Results", "The results collection where the analysis values should be stored."));
      #endregion 

      #region Create operators
      DataTableValuesCollector dataTableValuesCollector = new DataTableValuesCollector();
      ResultsCollector resultsCollector = new ResultsCollector();

      dataTableValuesCollector.CollectedValues.Add(new LookupParameter<IntValue>("EvaluatedSolutions", null, EvaluatedSolutionsParameter.Name));
      dataTableValuesCollector.DataTableParameter.ActualName = EvaluatedSolutionsHistoryParameter.Name;

      resultsCollector.CollectedValues.Add(new LookupParameter<DataTable>(EvaluatedSolutionsHistoryParameter.Name));
      resultsCollector.ResultsParameter.ActualName = ResultsParameter.Name;
      #endregion

      #region Create operator graph
      OperatorGraph.InitialOperator = dataTableValuesCollector;
      dataTableValuesCollector.Successor = resultsCollector;
      resultsCollector.Successor = null;
      #endregion
    }
  }
}